'use strict';

const stringUtil = require("./string-util");
const Generator = require('yeoman-generator');
const fs = require('fs');
let mapping;
let path;

module.exports = class extends Generator {

    constructor(args, opts) {
        super(args, opts);

        //reading and setting mapping file
        this.argument("mappingFile", {type: String, required: false, description: "Path to mapping file"});
        this.argument("projectPath", {
            type: String,
            required: false,
            description: "Path to project folder",
            default: ""
        });

        path = stringUtil.path(opts.projectPath);

        if (opts.mappingFile != null) {
            let data = fs.readFileSync(opts.mappingFile);
            mapping = JSON.parse(data);
        } else {
            mapping = require("../lib/config/mapping.json");
        }
    }

    /**
     * reads user inputs
     */
    async prompting() {
        this.answers = await this.prompt([
            //definition of inputs
            {
                type: "input",
                name: "jsonFile",
                message: "path to json file with entity definition",
            },
            {
                type: "input",
                name: "rootPackageName",
                message: "root package name",
            },
            {
                type: "input",
                name: "schemeName",
                message: "scheme name, format (three_word_scheme)",
            },
            {
                type: "input",
                name: "contextRoot",
                message: "URL context root of component"
            }
        ]);
    }

    /**
     * generete files from templates
     */
    async writing() {
        //reading input file
        const data = fs.readFileSync(this.answers.jsonFile);
        const json = JSON.parse(data);

        //util functions for data edition
        const dataUtil = require("./data-util");

        //creates names for db before changeing them to camel case
        dataUtil.createDbNames(json.atributes);
        dataUtil.editNamesToCamelCase(json.atributes);

        //edites types from EA to java types
        dataUtil.editTypesJava(json.atributes, mapping);

        //declaration of all constants used in templates
        //entity name in camel case
        const entityName = stringUtil.camelCase(json.entityName);
        //entity name in camel case first letter lower case
        const entityNameLower = stringUtil.nameLower(entityName);
        //entity name all lower case no spaces
        const entityNameAllLower = entityNameLower.toLocaleLowerCase();
        //entity name all lower case, words seperated with dashes
        const entityNameDash = stringUtil.dashName(json.entityName);
        //entity name all lower case, words seperated with underscores
        const entityDbName = stringUtil.underscoreName(json.entityName);
        //scheme name
        const schemeName = this.answers.schemeName.toLocaleLowerCase();
        //scheme name with no seperation between words
        const schemePackageName = stringUtil.nameFromDbScheme(schemeName);
        //root package sepereted with dots
        const rootPackageName = this.answers.rootPackageName;
        //root package seperated with slashes
        const rootPackagePath = stringUtil.packagePath(this.answers.rootPackageName);
        //endpoint path
        const contextRoot = stringUtil.path(this.answers.contextRoot);
        //java files

        //creates entity file
        this.fs.copyTpl(
            this.templatePath('Entity.java'),
            this.destinationPath(path + 'src/main/java/' + rootPackagePath + '/db/domain/' + schemePackageName + '/' + entityName + 'Ent.java'), {
                rootPackageName: rootPackageName,
                entityName: entityName,
                entityDbName: entityDbName,
                schemeName: schemePackageName,
                schemeDbName: schemeName,
                atributes: json.atributes,
                types: mapping.types
            }
        );

        //creates repository file
        this.fs.copyTpl(
            this.templatePath('Repository.java'),
            this.destinationPath(path + 'src/main/java/' + rootPackagePath + '/db/repo/' + schemePackageName + '/' + entityName + 'Repository.java'), {
                rootPackageName: rootPackageName,
                entityName: entityName,
                entityNameLower: entityNameLower,
                schemeName: schemePackageName,
            }
        );

        //creates mapper file
        this.fs.copyTpl(
            this.templatePath('Mapper.java'),
            this.destinationPath(path + 'src/main/java/' + rootPackagePath + '/modules/' + entityNameAllLower + '/api/' + entityName + 'Mapper.java'), {
                rootPackageName: rootPackageName,
                entityName: entityName,
                entityNameLower: entityNameLower,
                entityNameAllLower: entityNameAllLower,
                schemeName: schemePackageName,
            }
        );

        //creates serivce file
        this.fs.copyTpl(
            this.templatePath('Service.java'),
            this.destinationPath(path + 'src/main/java/' + rootPackagePath + '/modules/' + entityNameAllLower + '/' + entityName + 'ServiceImpl.java'), {
                rootPackageName: rootPackageName,
                entityName: entityName,
                entityNameLower: entityNameLower,
                entityNameAllLower: entityNameAllLower,
                schemeName: schemePackageName,
                schemeDbName: schemeName,
                atributes: json.atributes
            }
        );

        this.fs.copyTpl(
            this.templatePath('Test.java'),
            this.destinationPath(path + 'src/test/java/' + rootPackagePath + '/' + entityName + 'Test.java'), {
                rootPackageName: rootPackageName,
                entityName: entityName,
                contextRoot: contextRoot,
                entityNameDash: entityNameDash,
            }
        );

        //creates xml part that is necesery to add to pom.xml
        this.fs.copyTpl(
            this.templatePath('pom.txt'),
            this.destinationPath(path + 'pom.txt'), {
                rootPackageName: rootPackageName,
                entityNameAllLower: entityNameAllLower,
                entityNameDash: entityNameDash
            }
        )

        //chaneing type from yaml to db
        dataUtil.editTypesDb(json.atributes, mapping);

        this.fs.copyTpl(
            this.templatePath('changelog.txt'),
            this.destinationPath(path + 'changelog.txt'), {
                schemeName: schemeName,
                atributes: json.atributes,
                enitityDbName: entityDbName,
            }
        )

        //chaneing type from java to yaml
        dataUtil.editTypesYaml(json.atributes, mapping);

        //creates yaml file
        this.fs.copyTpl(
            this.templatePath('Endpoints.yaml'),
            this.destinationPath(path + 'src/main/resources/specifikace/api-' + entityNameDash + '.yaml'), {
                entityName: entityName,
                atributes: json.atributes,
                entityNameOriginal: json.entityName,
                entityNameDash: entityNameDash
            }
        )
    }

};
